// Golden ARMs demo
// /Mic, 2000 | stabmaster_@hotmail.com | come.to/progzone
//
//
// How to compile:
//  zarmasm -CPU ARM7TDMI -Littleend start.asm
//  zarmasm -CPU ARM7TDMI -Littleend garmdata.asm
//  zarmcc -c -Wall -Otime -ansic -fpu none -Littleend -cpu ARM7TDMI -apcs /narrow/noswst goldarms.c -o goldarms.o
//  zarmlink -bin -first start.o start.o goldarms.o garmdata.o -map -ro-base 0x08000000 -o goldarms.bin
//


#include <stdlib.h>
#include <math.h>
#include "gbal.h"
#include "meshes.h"


extern u8 lbl_font;
u8 *font=&lbl_font;
int font_width, font_height, char_size;


int camX,camY,camZ;

u16 ymul240[160];
float sine[256], cosine[256];

float pointsG[84];
float pointsB[66];
float pointsA[90];

long pointsG2D[56];
long pointsB2D[44];
long pointsA2D[60];

char message[348] = "Mic brings you another smash-hit for the GBA,   ca"\
                    "lled \'Golden ARMs\'...    powered by GBALib...   "\
                    "120 edges,  80 vertices,  one machine...      Be s"\
                    "ure to visit these sites:   www.gbadev.org     www"\
                    ".consoledev.com    www.zophar.net    come.to/progz"\
                    "one ....   and don\'t forget to /join #gbadev, the"\
                    " prime GBA development irc-channel @ EFNet.....   ";
u8 colors[347] =
{
 36,36,36,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
 4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,  4,4,4,4,4,20,20,20,20,20,20,20,20,20,20,20,20,20,4,4,
 4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,51,51,51,51,51,51,4,4,4,4,4,4,  36,36,36,4,4,4,4,
 4,4,4,4,4,36,36,4,4,4,4,4,4,4,4,4,4,4,4,36,36,36,4,4,4,4,4,4,4,4,4,4,4,4,4,
 4,4,4,4,4,4,4,4,  4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
 4,67,67,67,67,67,67,67,67,67,67,67,67,67,67,4,4,4,4,4,67,67,67,  67,67,67,67,67,67,67,67,67,67,67,67,
 67,67,67,4,4,4,4,67,67,67,67,67,67,67,67,67,67,67,67,67,67,4,4,4,4,67,67,67,67,67,67,67,67,67,67,
 67,67,67,  67,67,67,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,36,
 36,36,36,36,4,4,4,4,4,4,4,4,4,4,4,4,4,  4,4,4,4,4,4,4,51,51,51,4,4,4,4,4,4,4,4,
 4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,67,67,67,67,67,4,4,4,4,4,4,4,4
};





void Rotate(float *src, float *dest, int rx, int ry, int rz, int num_points)
{
int i, offs;
float xsa,xca, ysa,yca, zsa,zca;
float xr,yr,zr,f1;

 xsa = sine[rx&255];
 xca = cosine[rx&255];
 ysa = sine[ry&255];
 yca = cosine[ry&255];
 zsa = sine[rz&255];
 zca = cosine[rz&255];

 offs = 0;
 for (i=0; i<num_points; i++)
 {
  xr = src[offs];
  yr = src[offs+1];
  zr = src[offs+2];
  if (rx)
  { 
   f1 = yr;
    yr = ((xca*yr)-(xsa*zr));
    zr = ((xsa*f1)+(xca*zr));
  }
  if (ry)
  { 
    f1 = xr;
    xr = ((yca*xr)-(ysa*zr));
    zr = ((ysa*f1)+(yca*zr));
  }
  if (rz)
  { 
    f1 = xr;
    xr = ((zca*xr)-(zsa*yr));
    yr = ((zsa*f1)+(zca*yr));
  }
  dest[offs] = xr;
  dest[offs+1] = yr;
  dest[offs+2] = zr;
  offs += 3;
 }
}



void Project(float *src,long *dest,int num_points)
{
int i,si,di;
float sx,sy,sz;
  si = 0;
  di = 0;
  for (i=0; i<num_points; i++)
  {
    sx = src[si]; 
    sy = src[si+1];
    sz = src[si+2];
    si += 3;
    sz += (float)(camZ);
    if (sz!=0)
    {
      dest[di] = (long)((FOCAL*sx)/sz)+camX;
      dest[di+1] = (long)((FOCAL*sy)/sz)+camY;
    }else
    {
      dest[di] = camX;
      dest[di+1] = camY;
    }
    di += 2;
  }
}





void draw_line(long x1, long y1, long x2, long y2, u8 col)
{
int dx,dy,numpixels;
int d,dinc1,dinc2;
int xinc1,xinc2, yinc1,yinc2;
int x,y;
u8 *dest;
  dest = BACK;
  dx = (int)(x2-x1);
  dy = (int)(y2-y1);
  if (dx<0) { dx = -dx; }
  if (dy<0) { dy = -dy; }
  if (dx>dy)
  {
    numpixels = dx+1;
    d = (dy<<1)-dx;
    dinc1 = (dy<<1);
    dinc2 = (dy-dx)<<1;
    xinc1 = 1;
    xinc2 = 1;
    yinc1 = 0;
    yinc2 = 1;
  }else
  {
    numpixels = dy+1;
    d = (dx<<1)-dx;
    dinc1 = (dx<<1);
    dinc2 = (dx-dy)<<1;
    xinc1 = 0;
    xinc2 = 1;
    yinc1 = 1;
    yinc2 = 1; 
  }
  if (x1>x2) { xinc1 = -xinc1; xinc2 = -xinc2; }
  if (y1>y2) { yinc1 = -yinc1; yinc2 = -yinc2; }
  x = (int)x1;
  y = (int)y1;
  for (; numpixels; numpixels--)
  {
    if ((y>=0)&&(y<160))
    {
      if ((x>=0)&&(x<240))
      {
        *(dest+ymul240[y]+x) = col;
      }
    }
    if (d<0)
    {
      d += dinc1;
      x += xinc1;
      y += yinc1;
    }else  
    {
      d += dinc2;
      x += xinc2;
      y += yinc2;
    }
  }
}



void put_char_spec(int x, int y, u8 col, u8 tecken)
{
u8 *src,*dest;
int xdim,ydim;

  if (tecken==' ') { return; }

  src = (font+(char_size*tecken))+260;
  xdim = *(u8 *)(font+tecken+4);
  ydim = font_height;

  if ((x>239)||((x+xdim-1)<0)) { return; }
  if ((y>159)||((y+ydim-1)<0)) { return; }

  if (y<0) { src -= (y*xdim); ydim += y; y=0; }
  if ((y+ydim)>160) { ydim -= ((y+ydim)-160); }

  if (x<0) { src -= x; xdim += x; x=0; }
  if ((x+xdim)>240) { xdim -= ((x+xdim)-240); }

  dest = (u8 *)(BACK+(ymul240[y])+x);

 
  __asm 
  {
    mov r0,src
    mov r1,dest
    mov r2,ydim
    mov r3,font_width
    mov r5,xdim
    sub r3,r3,r5
    mov r4,#240
    sub r4,r4,r5
    mov r7,col
    pcYloop:
      mov r5,xdim
      pcXloop:
        ldrb r6,[r0]
        orrs r6,r6,r6
        beq invisible
          strb r7,[r1]
        invisible:
        add r0,r0,#1
        add r1,r1,#1
        subs r5,r5,#1
      bne pcXloop
      add r0,r0,r3  
      add r1,r1,r4
      add r7,r7,#1
      subs r2,r2,#1
    bne pcYloop
  }
}

 

 
int GBA_main()
{
int i, p1,p2;

u8 a1=0,a2,a3=0;
int xpos,xpos2=240, ypos;
int min_xpos;

float f;
u8 thetaG = 0,
   thetaB = 45,
   thetaA = 22;


  for (i=0; i<160; i++)
  {
    ymul240[i] = i*240;
  }

  font_width = *(u8 *)(font+2);
  font_height = *(u8 *)(font+3);
  char_size = (font_width*font_height);
 
  min_xpos = -(348*font_width);


  for (f=0; f<256; f+=1)
  {
    sine[(int)f] = (float)sin(f*3.1415926f/128.0f);
    cosine[(int)f] = (float)cos(f*3.1415926f/128.0f);
  }

  gbalSetMode(_240x160x1x256_);
  gbalSetColor(BACKGROUND, 1, gbalMakeColor(255,70,70));
  gbalSetColor(BACKGROUND, 2, gbalMakeColor(70,255,70));
  gbalSetColor(BACKGROUND, 3, gbalMakeColor(70,70,255));

  for (i=1; i<16; i++)
  {
    gbalSetColor(BACKGROUND, i+3,  (i<<5)|(i+15));
    gbalSetColor(BACKGROUND, i+19, ((i+15)<<5)|(i+15));
    gbalSetColor(BACKGROUND, i+35, (i<<10)|(i<<5)|30);
    gbalSetColor(BACKGROUND, i+51, (30<<10)|(i<<5));
    gbalSetColor(BACKGROUND, i+67, ((i+15)<<5));
  }




  while (1)
  {
    // erase
    gbalClearBuffer(0);

    // move the "camera" (viewer) back and forth..
    camZ = (int)(sine[a3]*90)+290;
    a3 += 1;

    camX = 65; camY = 46; 
    Rotate(&_g_[0], &pointsG[0], thetaG,0,thetaG, 28);
    Project(&pointsG[0], &pointsG2D[0], 28);

    camX = 125; camY = 54; 
    Rotate(&_b_[0], &pointsB[0], 0,thetaB,thetaB, 22);
    Project(&pointsB[0], &pointsB2D[0], 22);

    camX = 175; camY = 50; 
    Rotate(&_a_[0], &pointsA[0], thetaA,thetaA,thetaA, 30);
    Project(&pointsA[0], &pointsA2D[0], 30);

    thetaG++;
    thetaB++;
    thetaA++;


    // draw the 'g'
    for (i=0; i<14; i++)
    {
      p1 = _g_lines_[i+i]<<1;
      p2 = _g_lines_[i+i+1]<<1;
      draw_line(pointsG2D[p1],pointsG2D[p1+1], pointsG2D[p2],pointsG2D[p2+1], 1);
      draw_line(pointsG2D[p1+28],pointsG2D[p1+29], pointsG2D[p2+28],pointsG2D[p2+29], 1);
      draw_line(pointsG2D[p1],pointsG2D[p1+1], pointsG2D[p1+28],pointsG2D[p1+29], 1);
    }

    // draw the 'b'
    for (i=0; i<11; i++)
    {
      p1 = _b_lines_[i+i]<<1;
      p2 = _b_lines_[i+i+1]<<1;
      draw_line(pointsB2D[p1],pointsB2D[p1+1], pointsB2D[p2],pointsB2D[p2+1], 2);
      draw_line(pointsB2D[p1+22],pointsB2D[p1+23], pointsB2D[p2+22],pointsB2D[p2+23], 2);
      draw_line(pointsB2D[p1],pointsB2D[p1+1], pointsB2D[p1+22],pointsB2D[p1+23], 2);
    }

    // draw the 'a'
    for (i=0; i<15; i++)
    {
      p1 = _a_lines_[i+i]<<1;
      p2 = _a_lines_[i+i+1]<<1;
      draw_line(pointsA2D[p1],pointsA2D[p1+1], pointsA2D[p2],pointsA2D[p2+1], 3);
      draw_line(pointsA2D[p1+30],pointsA2D[p1+31], pointsA2D[p2+30],pointsA2D[p2+31], 3);
      draw_line(pointsA2D[p1],pointsA2D[p1+1], pointsA2D[p1+30],pointsA2D[p1+31], 3);
    }
   

    // draw the scrolling text
    a2 = a1;
    xpos = xpos2;
    for (i=0; i<347; i++)
    {
      ypos = (int)(sine[a2]*6)+130;
      put_char_spec(xpos, ypos, colors[i], message[i]);
      xpos += 8;
      a2 += 21;
    }
    
    // move the scrolling text
    a1 += 2;
    xpos2 -= 1;
    if (xpos2<min_xpos) { xpos2=240; }

    // flip the pages
    gbalSwapBuffers();
  }


  return 0;
}

       





